function AEventListener(type, listener, object, add, useCapture)
{
	this.type = type;
	this.listener = listener;
	this.object = object;
	this.useCapture = useCapture || false;

	var _this = this;

	function _listener(e)
	{
		_this._handleEvent(e);
		_this.listener(e);
	}

	this._listener = _listener;

	this._on_type = 'on' + type;
	this._added = false;

	if (add || false) {
		this.add();
	}
}

AEventListener.prototype =
{
	add: function()
	{
		if (!this._added) {
			this._add();
			this._added = true;
		}
	},

	remove: function()
	{
		if (this._added) {
			this._remove();
			this._added = false;
		}
	},

	_handleEvent: function(e)
	{
		// nop
	}
};

if (document.addEventListener) {

	// DOM functions
	AEventListener.prototype._add = function()
	{
		this.object.addEventListener(this.type, this._listener, this.useCapture);
	};

	AEventListener.prototype._remove = function()
	{
		this.object.removeEventListener(this.type, this._listener, this.useCapture);
	};

} else if (document.attachEvent) {

	// IE functions
	function add()
	{
		this.object.attachEvent(this._on_type, this._listener);
	}
	AEventListener.prototype._add = add;

	function remove()
	{
		this.object.detachEvent(this._on_type, this._listener);
	}
	AEventListener.prototype._remove = remove;

	function handleEvent(e)
	{
		var type = e.type;

		e.charCode = ((type == 'keypress') ? e.keyCode : 0);
		e.isChar = (e.charCode > 0);
		e.target = e.srcElement;

		e.preventDefault = function ()
		{
			this.returnValue = false;
		};

		e.stopPropagation = function ()
		{
			this.cancelBubble = true;
		};

		var root = document.body;
		if (!root) {
			root = document.documentElement;
		}

		e.pageX = e.clientX + root.scrollLeft;
		e.pageY = e.clientY + root.scrollTop;

		if (type == 'mouseout') {
			e.relatedTarget = e.toElement;
		} else if (type == 'mouseover') {
			e.relatedTarget = e.fromElement;
		}
	}
	AEventListener.prototype._handleEvent = handleEvent;

} else {

	// Old browsers functions
	AEventListener.prototype._add = function()
	{
		this.object[this._on_type] = this._listener;
	};

	AEventListener.prototype._remove = function()
	{
		this.object[this._on_type] = null;
	};
}
